@@ -22,6 +22,108 @@ perf_event_damon_aggregated = 'damon:damon_aggregated'
 perf_event_damon_monitor_intervals_tune = 'damon:damon_monitor_intervals_tune'
 perf_event_damos_before_apply = 'damon:damos_before_apply'
 
+class DamonIdleMsPercentile:
+    percentile = None
+    idle_ms = None
+
+    def __init__(self, percentile, idle_ms):
+        self.percentile = _damo_fmt_str.text_to_nr(percentile)
+        self.idle_ms = _damo_fmt_str.text_to_ms(idle_ms)
+
+    @classmethod
+    def from_kvpairs(cls, kv):
+        return DamonIdleMsPercentile(
+                percentile=kv['percentile'], idle_ms=kv['idle_ms'])
+
+    def to_kvpairs(self, raw=False):
+        return collections.OrderedDict([
+            ('percentile', _damo_fmt_str.format_nr(self.percentile, raw)),
+            ('idle_ms', _damo_fmt_str.format_time_ms_exact(self.idle_ms, raw)),
+            ])
+
+class DamonIdleMsPercentiles:
+    percentile_ms_list = None  # list of DamonIdleMsPercentile
+
+    def __init__(self, percentile_ms_list):
+        self.percentile_ms_list = percentile_ms_list
+
+    @classmethod
+    def from_kvpairs(cls, kv):
+        percentile_ms_list = [DamonIdleMsPercentile.from_kvpairs(x)
+                              for x in kv]
+        return DamonIdleMsPercentiles(percentile_ms_list=percentile_ms_list)
+
+    def to_kvpairs(self, raw=False):
+        return [x.to_kvpairs(raw) for x in self.percentile_ms_list]
+
+class DamonStatSnapshot:
+    idletime_ms_percentiles = None  # DamonIdleMsPercentiles
+    memory_bw_bytes_per_sec = None   # bytes per second
+    aggr_interval_us = None # aggregation interval, available from 6.18
+
+    def __init__(self, idletime_ms_percentiles,
+                 memory_bw_bytes_per_sec, aggr_interval_us):
+        self.idletime_ms_percentiles = idletime_ms_percentiles
+        self.memory_bw_bytes_per_sec = _damo_fmt_str.text_to_bytes(
+                memory_bw_bytes_per_sec)
+        # aggr_interval_us on DAMON_STAT is available since 6.18
+        if aggr_interval_us is not None:
+            self.aggr_interval_us = _damo_fmt_str.text_to_us(aggr_interval_us)
+
+    @classmethod
+    def from_kvpairs(cls, kv):
+        return DamonStatSnapshot(
+                idletime_ms_percentiles=DamonIdleMsPercentiles.from_kvpairs(
+                    kv['idletime_ms_percentiles']),
+                memory_bw_bytes_per_sec=kv['memory_bw_bytes_per_sec'],
+                aggr_interval_us=kv['aggr_interval_us'])
+
+    def to_kvpairs(self, raw=False):
+        if self.aggr_interval_us is not None:
+            aggr_interval_us = _damo_fmt_str.format_time_us_exact(
+                    self.aggr_interval_us, raw)
+        else:
+            aggr_interval_us = None
+
+        return collections.OrderedDict([
+            ('idletime_ms_percentiles',
+             self.idletime_ms_percentiles.to_kvpairs(raw)),
+            ('memory_bw_bytes_per_sec',
+             _damo_fmt_str.format_sz(self.memory_bw_bytes_per_sec, raw)),
+            ('aggr_interval_us', aggr_interval_us),
+            ])
+
+    @classmethod
+    def capture(cls, live_only=False):
+        damon_stat_parm_dir = '/sys/module/damon_stat/parameters'
+        if not os.path.isdir(damon_stat_parm_dir):
+            return None, 'damon_stat unsupported'
+        if live_only:
+            with open(os.path.join(damon_stat_parm_dir, 'enabled'), 'r') as f:
+                if f.read().strip().lower() == 'n':
+                    return None, 'damon_stat is not enabled'
+        with open(os.path.join(damon_stat_parm_dir,
+                               'memory_idle_ms_percentiles'), 'r') as f:
+            idletime_ms_percentiles = DamonIdleMsPercentiles(
+                    percentile_ms_list=[
+                        DamonIdleMsPercentile(percentile, idle_ms)
+                        for percentile, idle_ms in
+                        enumerate(f.read().strip().split(','))])
+        with open(os.path.join(damon_stat_parm_dir,
+                               'estimated_memory_bandwidth'), 'r') as f:
+            memory_bw_bytes_per_sec = int(f.read())
+        aggr_interval_file = os.path.join(damon_stat_parm_dir,
+                                          'aggr_interval_us')
+        if not os.path.isfile(aggr_interval_file):
+            aggr_interval_us = None
+        else:
+            with open(aggr_interval_file, 'r') as f:
+                aggr_interval_us = int(f.read())
+        return DamonStatSnapshot(
+                idletime_ms_percentiles=idletime_ms_percentiles,
+                memory_bw_bytes_per_sec=memory_bw_bytes_per_sec,
+                aggr_interval_us=aggr_interval_us), None
+
 class DamonSnapshot:
     '''
     Contains a snapshot of data access monitoring results
@@ -56,6 +158,7 @@ class DamonSnapshot:
         if 'sample_interval_us' in kv and kv['sample_interval_us'] is not None:
             sample_interval_us = _damo_fmt_str.text_to_us(
                     kv['sample_interval_us'])
+
         return DamonSnapshot(
                 _damo_fmt_str.text_to_ns(kv['start_time']),
                 _damo_fmt_str.text_to_ns(kv['end_time']),
@@ -85,6 +188,9 @@ class DamonSnapshot:
             ('sample_interval_us', sample_interval_us),
             ])
 
+record_data_source_unknown = 'unknown'
+record_data_source_damon_stat = 'damon_stat'
+
 class DamonRecord:
     '''
     Contains data access monitoring results for single target
@@ -96,9 +202,10 @@ class DamonRecord:
     target_id = None
     scheme_filters = None
     snapshots = None
+    data_source = None # source of data that used to generate this.
 
     def __init__(self, kd_idx, ctx_idx, intervals, scheme_idx, target_id,
-                 scheme_filters):
+                 scheme_filters, data_source=record_data_source_unknown):
         self.kdamond_idx = kd_idx
         self.context_idx = ctx_idx
         self.intervals = intervals
@@ -106,6 +213,7 @@ class DamonRecord:
         self.target_id = target_id
         self.scheme_filters = scheme_filters
         self.snapshots = []
+        self.data_source = data_source
 
     @classmethod
     def from_kvpairs(cls, kv):
@@ -115,6 +223,11 @@ class DamonRecord:
                 kv[keyword] = None
         if not 'scheme_filters' in kv:
             kv['scheme_filters'] = []
+        # data_source was introduced after v2.9.8
+        if not 'data_source' in kv:
+            data_source = record_data_source_unknown
+        else:
+            data_source = kv['data_source']
 
         record = DamonRecord(
                 kv['kdamond_idx'], kv['context_idx'],
@@ -122,7 +235,7 @@ class DamonRecord:
                 if kv['intervals'] is not None else None,
                 kv['scheme_idx'], kv['target_id'],
                 [_damon.DamosFilter.from_kvpairs(pairs) for pairs in
-                 kv['scheme_filters']])
+                 kv['scheme_filters']], data_source=data_source)
         record.snapshots = [DamonSnapshot.from_kvpairs(s)
                 for s in kv['snapshots']]
 
@@ -142,6 +255,7 @@ class DamonRecord:
         else:
             ordered_dict['scheme_filters'] = []
         ordered_dict['snapshots'] = [s.to_kvpairs(raw) for s in self.snapshots]
+        ordered_dict['data_source'] = self.data_source
         return ordered_dict
 
     def can_merge(self, other):
@@ -944,57 +1058,9 @@ def load_proc_stats(filepath):
         kvpairs = json.load(f)
     return [ProcStatsSnapshot.from_kvpairs(x) for x in kvpairs]
 
-def add_childs_target(kdamonds):
-    # TODO: Support multiple kdamonds
-    if not _damon.target_has_pid(kdamonds[0].contexts[0].ops):
-        return
-    current_targets = kdamonds[0].contexts[0].targets
-
-    for target in current_targets:
-        if target.pid is None:
-            continue
-        try:
-            childs_pids = subprocess.check_output(
-                    ['ps', '--ppid', '%s' % target.pid, '-o', 'pid=']
-                    ).decode().split()
-        except:
-            childs_pids = []
-        if len(childs_pids) == 0:
-            continue
-
-        # TODO: Commit all at once, out of this loop
-        new_targets = []
-        for child_pid in childs_pids:
-            # skip the child if already in the targets
-            if child_pid in ['%s' % t.pid for t in current_targets]:
-                continue
-            # remove already terminated targets, since committing already
-            # terminated targets to DAMON fails
-            new_targets = [target for target in current_targets
-                    if pid_running('%s' % target.pid)]
-            new_targets.append(_damon.DamonTarget(pid=child_pid, regions=[]))
-        if new_targets == []:
-            continue
-
-        # commit the new set of targets
-        kdamonds[0].contexts[0].targets = new_targets
-        err = _damon.commit(kdamonds, commit_targets_only=True)
-        if err is not None:
-            kdamonds[0].contexts[0].targets = current_targets
-            return 'commit failed (%s)' % err
-    return None
-
-def pid_running(pid):
-    '''pid should be string'''
-    try:
-        subprocess.check_output(['ps', '--pid', pid])
-        return True
-    except:
-        return False
-
 def all_targets_terminated(targets):
     for target in targets:
-        if pid_running('%s' % target.pid):
+        if _damon.pid_running('%s' % target.pid):
             return False
     return True
 
@@ -1013,8 +1079,8 @@ def poll_target_pids(kdamonds):
 
 class RecordingHandle:
     '''
-    Specifies the recording request.  The recording can be started by
-    start_recording(), and finished by finish_recording().
+    Recording requests, status and intermediate results.  The recording can be
+    started by start_recording(), and finished by finish_recording().
 
     As a result of the two function calls, below files can be generated.
 
@@ -1047,8 +1113,9 @@ class RecordingHandle:
     perf_pipe = None
 
     # for access patterns snapshot
-    snapshot_request = None
-    snapshot_records = None
+    snapshot_request = None # SnapshotRequest object.
+    snapshot_records = None # list of DamonRecord objects retrieved via
+                            # get_snapshot_records_of()
     snapshot_count = None
     snapshot_interval_sec = None
 
@@ -1143,7 +1210,7 @@ def start_recording(handle):
     while (poll_target_pids(handle.kdamonds) or
            _damon.any_kdamond_running()):
         if handle.add_child_tasks is True:
-            add_childs_target(handle.kdamonds)
+            _damon.add_childs_target(handle.kdamonds)
 
         if handle.mem_footprint_snapshots is not None:
             record_mem_footprint(handle.kdamonds,
@@ -1272,22 +1339,28 @@ def can_merge(left_region, right_region):
             left_region.nr_accesses == right_region.nr_accesses and
             left_region.age == right_region.age)
 
+def merged_regions(regions):
+    merged = []
+
+    for region in regions:
+        if len(merged) > 0:
+            last_region = merged[-1]
+            if can_merge(last_region, region):
+                last_region.end = region.end
+                if last_region.sz_filter_passed is not None:
+                    last_region.sz_filter_passed += region.sz_filter_passed
+                continue
+        merged.append(region)
+    return merged
+
 def tried_regions_to_snapshot(scheme, intervals, merge_regions):
     snapshot_end_time_ns = time.time() * 1000000000
     snapshot_start_time_ns = snapshot_end_time_ns - intervals.aggr * 1000
-    regions = []
 
-    for tried_region in scheme.tried_regions:
-        '''Merge regions that having same access pattern, since DAMON usually
-        splits regions unnecessarily to keep the min_nr_regions'''
-        if merge_regions and len(regions) > 0:
-            last_region = regions[-1]
-            if can_merge(last_region, tried_region):
-                last_region.end = tried_region.end
-                if last_region.sz_filter_passed is not None:
-                    last_region.sz_filter_passed += tried_region.sz_filter_passed
-                continue
-        regions.append(tried_region)
+    if merge_regions:
+        regions = merged_regions(scheme.tried_regions)
+    else:
+        regions = [r for r in scheme.tried_regions]
     if scheme.tried_bytes is not None:
         total_bytes = scheme.tried_bytes
     else:
@@ -1317,66 +1390,6 @@ def tried_regions_to_records_of(idxs, merge_regions):
                 break
     return records
 
-def three_regions_of(pid):
-    '''
-    Return three big mapped virtual address ranges of a given process, which
-    separated by the two huge gaps[1].
-
-    [1] https://docs.kernel.org/mm/damon/design.html#vma-based-target-address-range-construction
-    '''
-    if not os.path.isfile('/proc/%s/maps' % pid):
-        print('maps file for %s pid not found' % pid)
-        exit(0)
-    with open('/proc/%s/maps' % pid, 'r') as f:
-        maps_content = f.read()
-    regions = []
-    for line in maps_content.split('\n'):
-        if line == '':
-            continue
-        start, end = [int(addr, 16) for addr in line.split()[0].split('-')]
-        if len(regions) > 0 and regions[-1].end == start:
-            regions[-1].end = end
-        else:
-            regions.append(_damon.DamonRegion(start, end))
-
-    gaps = []
-    for idx, region in enumerate(regions):
-        if idx == 0:
-            continue
-        prev_region = regions[idx - 1]
-        if region.start != prev_region.end:
-            gaps.append([prev_region.end, region.start])
-    gaps.sort(key=lambda x: x[1] - x[0], reverse=True)
-    if len(gaps) < 2:
-        return regions
-    # sort biggest two gaps in address
-    gaps = sorted(gaps[:2], key=lambda x: x[0])
-
-    return [_damon.DamonRegion(regions[0].start, gaps[0][0]),
-            _damon.DamonRegion(gaps[0][1], gaps[1][0]),
-            _damon.DamonRegion(gaps[1][1], regions[-1].end)]
-
-def install_target_regions_if_needed(kdamonds):
-    '''Returns an error string, or None'''
-    need_install = False
-    for kd in kdamonds:
-        for ctx in kd.contexts:
-            if ctx.ops != 'vaddr':
-                continue
-            need_install = True
-            for target in ctx.targets:
-                target.regions = three_regions_of(target.pid)
-    if not need_install:
-        return None
-    err = _damon.commit(kdamonds)
-    for kd in kdamonds:
-        for ctx in kd.contexts:
-            if ctx.ops != 'vaddr':
-                continue
-            for target in ctx.targets:
-                target.regions = []
-    return err
-
 def update_get_snapshot_records(kdamond_idxs, scheme_idxs,
         total_sz_only, merge_regions):
     if total_sz_only:
@@ -1436,10 +1449,6 @@ def get_snapshot_records(monitor_scheme, total_sz_only, merge_regions):
 
     orig_kdamonds = _damon.current_kdamonds()
 
-    err = install_target_regions_if_needed(orig_kdamonds)
-    if err is not None:
-        return None, 'vaddr region install failed (%s)' % err
-
     installed, idxs, updated_kdamonds, err = find_install_scheme(
             monitor_scheme)
     if err:
@@ -1580,10 +1589,121 @@ def filter_records_by_temperature(records, temperature_ranges,
             snapshot.regions = filtered_regions
             snapshot.update_total_bytes()
 
+class SnapshotRequest:
+    '''
+    Request for getting single snapshot records from running kdamonds.
+    '''
+    tried_regions_of = None
+
+    # filter to be applied using DAMOS filters, if available.
+    snapshot_damos_filters = None
+
+    # filter to be applied on retrieved snapshot, by damo.
+    record_filter = None
+
+    # more detailed requests
+    total_sz_only = None
+    dont_merge_regions = None
+
+    def __init__(
+            self, tried_regions_of=None, snapshot_damos_filters=None,
+            record_filter=None, total_sz_only=False, dont_merge_regions=True):
+        self.tried_regions_of = tried_regions_of
+        self.snapshot_damos_filters = snapshot_damos_filters
+        self.record_filter = record_filter
+        self.total_sz_only = total_sz_only
+        self.dont_merge_regions = dont_merge_regions
+
+def should_get_snapshot_from_damon_stat(request):
+    if request.tried_regions_of is not None:
+        return False
+    if request.snapshot_damos_filters:
+        return False
+
+    if _damon.any_kdamond_running():
+        return False
+
+    param_dir = '/sys/module/damon_stat/parameters/'
+    aggr_interval_us_file = os.path.join(param_dir, 'aggr_interval_us')
+    if not os.path.isfile(aggr_interval_us_file):
+        return False
+    enabled_file = os.path.join(param_dir, 'enabled')
+    with open(enabled_file, 'r') as f:
+        if f.read().strip().lower() == 'n':
+            return False
+
+    return True
+
+def read_damon_stat_param(param_name):
+    file_path = os.path.join('/sys/module/damon_stat/parameters', param_name)
+    with open(file_path, 'r') as f:
+        return f.read().strip()
+
+def get_snapshot_records_of_damon_stat(request):
+    aggr_interval_us = int(read_damon_stat_param('aggr_interval_us'))
+    sample_interval_us = aggr_interval_us / 20
+    snapshot_intervals = _damon.DamonIntervals(
+            sample=sample_interval_us, aggr=aggr_interval_us,
+            ops_update=60* 1000 * 1000,
+            intervals_goal=_damon.DamonIntervalsGoal(
+                access_bp=400, aggrs=3, min_sample_us=5000,
+                max_sample_us=10000000))
+
+    record = DamonRecord(kd_idx=-1, ctx_idx=0, intervals=snapshot_intervals,
+                         scheme_idx=None, target_id=0, scheme_filters=[],
+                         data_source=record_data_source_damon_stat)
+
+    snapshot_end_time_ns = time.time() * 1000000000
+    snapshot_start_time_ns = snapshot_end_time_ns - aggr_interval_us * 1000
+    idle_ms_percentiles = read_damon_stat_param('memory_idle_ms_percentiles')
+    idle_ms_percentiles = [int(x) for x in idle_ms_percentiles.split(',')]
+
+    mem_total_bytes = None
+    with open('/proc/meminfo', 'r') as f:
+        for line in f:
+            fields = line.split()
+            if fields[0] != 'MemTotal:':
+                continue
+            mem_total_bytes = int(fields[1]) * 1024
+    if mem_total_bytes is None:
+        return None, 'MemTotal retrieval fail'
+
+    regions = []
+    for percentile in range(100):
+        idle_ms_min = idle_ms_percentiles[percentile]
+        idle_ms_max = idle_ms_percentiles[percentile + 1]
+        idle_ms = int((idle_ms_min + idle_ms_max) / 2)
+
+        if idle_ms < 0:
+            nr_accesses_sample = 10
+        else:
+            nr_accesses_sample = 0
+
+        region = _damon.DamonRegion(
+                start=int(percentile * mem_total_bytes / 100),
+                end=int((percentile + 1) * mem_total_bytes / 100),
+                nr_accesses=nr_accesses_sample,
+                nr_accesses_unit=_damon.unit_samples,
+                age=idle_ms * 1000, age_unit=_damon.unit_usec,
+                sz_filter_passed=0)
+        regions.append(region)
+    if not request.dont_merge_regions:
+        regions = merged_regions(regions)
+
+    snapshot = DamonSnapshot(
+            start_time=snapshot_start_time_ns, end_time=snapshot_end_time_ns,
+            regions=regions, total_bytes=mem_total_bytes, damos_stats=None,
+            sample_interval_us=sample_interval_us)
+
+    record.snapshots = [snapshot]
+    return [record], None
+
 def get_snapshot_records_of(request):
     '''
     get records containing single snapshot from running kdamonds
     '''
+    if should_get_snapshot_from_damon_stat(request):
+        return get_snapshot_records_of_damon_stat(request)
     if request.tried_regions_of is None:
         access_pattern = _damon.DamosAccessPattern()
         filters = []
@@ -1679,40 +1799,9 @@ def set_snapshot_damos_filters_option(parser):
                 'Format is same to --damos_filter.'
                 ]))
 
-class RecordGetRequest:
-    # TODO: Extend to be used for recording
-
-    # source of the record.  If both are None, get snapshot
-    tried_regions_of = None
-    record_file = None
-
-    snapshot_damos_filters = None
-
-    record_filter = None
-
-    # more detailed requests
-    total_sz_only = None
-    dont_merge_regions = None
-
-    def __init__(
-            self, tried_regions_of=None, record_file=None,
-            snapshot_damos_filters=None,
-            record_filter=None,
-            total_sz_only=False, dont_merge_regions=True):
-        self.tried_regions_of = tried_regions_of
-        self.record_file = record_file
-        self.snapshot_damos_filters = snapshot_damos_filters
-        self.record_filter = record_filter
-        self.total_sz_only = total_sz_only
-        self.dont_merge_regions = dont_merge_regions
-
 def get_records(tried_regions_of=None, record_file=None,
                 snapshot_damos_filters=None, record_filter=None,
                 total_sz_only=False, dont_merge_regions=True):
-    request = RecordGetRequest(
-            tried_regions_of, record_file, snapshot_damos_filters,
-            record_filter, total_sz_only, dont_merge_regions)
-
     # If record is live snapshot, access pattern filtering is applied with
     # get_snapshot_records_of() because it uses DAMOS to get the snapshot.  If
     # the kernel has schemes_filters_addr feature, address ranges filter is
@@ -1725,8 +1814,11 @@ def get_records(tried_regions_of=None, record_file=None,
     else:
         filter_copy = RecordFilter(None, None, None, None, None, None, None)
 
-    if request.record_file is None:
-        records, err = get_snapshot_records_of(request)
+    if record_file is None:
+        records, err = get_snapshot_records_of(
+                SnapshotRequest(
+                    tried_regions_of, snapshot_damos_filters,
+                    record_filter, total_sz_only, dont_merge_regions))
         if err is not None:
             return None, err
         filter_copy.access_pattern = None
@@ -1734,10 +1826,10 @@ def get_records(tried_regions_of=None, record_file=None,
             filter_copy.address_ranges = None
             filter_copy.snapshot_time_ranges = None
     else:
-        if type(request.record_file) is not list:
-            request.record_file = [request.record_file]
+        if type(record_file) is not list:
+            record_file = [record_file]
         records = []
-        for record_file in request.record_file:
+        for record_file in record_file:
             if not os.path.isfile(record_file):
                 return None, '%s not found' % record_file
 
