@@ -1053,7 +1053,7 @@ class Damos:
             if self.apply_interval_us != 0 else 'per aggr interval')
         return ' '.join(action_words)
 
-    def to_str(self, raw, params_only=False):
+    def to_str(self, raw, params_only=False, omit_tried_regions=False):
         lines = [self.str_action_line(raw)]
         if self.access_pattern is not None:
             lines.append('target access pattern')
@@ -1076,7 +1076,8 @@ class Damos:
         if params_only is False and self.stats is not None:
             lines.append('statistics')
             lines.append(_damo_fmt_str.indent_lines(self.stats.to_str(raw), 4))
-        if params_only is False and self.tried_regions is not None:
+        if params_only is False and omit_tried_regions is False and \
+                self.tried_regions is not None:
             lines.append('tried regions (%s)' % _damo_fmt_str.format_sz(
                     self.tried_bytes, raw))
             for region in self.tried_regions:
@@ -1178,7 +1179,7 @@ class DamonCtx:
         for scheme in self.schemes:
             scheme.context = self
 
-    def to_str(self, raw, params_only=False):
+    def to_str(self, raw, params_only=False, omit_damos_tried_regions=False):
         ops_line_tokens = ['ops: %s' % self.ops]
         if self.ops_attrs.use_reports:
             ops_line_tokens.append('use_reports')
@@ -1201,7 +1202,7 @@ class DamonCtx:
         for idx, scheme in enumerate(self.schemes):
             lines.append('scheme %d' % idx)
             lines.append(_damo_fmt_str.indent_lines(
-                scheme.to_str(raw, params_only), 4))
+                scheme.to_str(raw, params_only, omit_damos_tried_regions), 4))
         return '\n'.join(lines)
 
     def __str__(self):
@@ -1270,7 +1271,8 @@ class Kdamond:
             words.append('cpu usage: %s' % self.get_cpu_usage())
         return ', '.join(words)
 
-    def to_str(self, raw, show_cpu=False, params_only=False):
+    def to_str(self, raw, show_cpu=False, params_only=False,
+               omit_damos_tried_regions=False):
         lines = []
         summary_line = self.summary_str(show_cpu, params_only,
                                         omit_defaults=False, raw_number=raw)
@@ -1279,7 +1281,7 @@ class Kdamond:
         for idx, ctx in enumerate(self.contexts):
             lines.append('context %d' % idx)
             lines.append(_damo_fmt_str.indent_lines(
-                ctx.to_str(raw, params_only), 4))
+                ctx.to_str(raw, params_only, omit_damos_tried_regions), 4))
         return '\n'.join(lines)
 
     def __str__(self):
@@ -1670,6 +1672,82 @@ def update_schemes_status(stats=True, tried_regions=True,
 
     return None
 
+def get_childs_pids(pid):
+    try:
+        childs_pids = subprocess.check_output(
+                ['ps', '--ppid', pid, '-o', 'pid=']
+                ).decode().split()
+    except:
+        childs_pids = []
+
+    ret = childs_pids
+    for child_pid in childs_pids:
+        childs_childs_pids = get_childs_pids(child_pid)
+        ret.extend(childs_childs_pids)
+
+    return ret
+
+def pid_running(pid):
+    '''pid should be string'''
+    try:
+        subprocess.check_output(['ps', '--pid', pid])
+        return True
+    except:
+        return False
+
+def add_childs_target(kdamonds):
+    # TODO: Support multiple kdamonds
+    if not target_has_pid(kdamonds[0].contexts[0].ops):
+        return
+    old_targets = kdamonds[0].contexts[0].targets
+    current_target_pids = []
+    new_target_pids = []
+
+    for target in old_targets:
+        if target is None:
+            continue
+        current_target_pids.append('%s' % target.pid)
+
+    for target_pid in current_target_pids:
+        # If this is already in new_target_pids, we have already recursively
+        # searched for its children as well.
+        if target_pid in new_target_pids:
+            continue
+        # Don't consider already terminated targets since committing already
+        # terminated targets to DAMON fails
+        if not pid_running(target_pid):
+            continue
+
+        new_target_pids.append(target_pid)
+        childs_pids = get_childs_pids(target_pid)
+        if len(childs_pids) == 0:
+            continue
+
+        for child_pid in childs_pids:
+            # skip the child if already in the targets
+            if child_pid in new_target_pids:
+                continue
+            new_target_pids.append(child_pid)
+
+    if set(new_target_pids) == set(current_target_pids):
+        return None
+    # Commit the new set of targets.
+    #
+    # Set no target region because it will instruct DAMON to keep current
+    # monitoring results (regions with nr_accesses, age, etc) for the target if
+    # exists.
+    #
+    # TODO: adjust the orders of the new targets so that only appropriate past
+    # monitoring results are inherited.
+    kdamonds[0].contexts[0].targets = [DamonTarget(pid = p, regions=[])
+                                       for p in new_target_pids]
+    err = commit(kdamonds, commit_targets_only=True)
+    if err is not None:
+        kdamonds[0].contexts[0].targets = old_targets
+        return 'commit failed (%s)' % err
+
+    return None
+
 def turn_damon_on(kdamonds_idxs):
     err = _damon_fs.turn_damon_on(kdamonds_idxs)
     if err:
